do

	--[[-----------------------------------------------------------------------------
	Button Widget
	Graphical Button.
	-------------------------------------------------------------------------------]]
	local Type, Version = "FrameAnchorDropdown", 2
	local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
	if not AceGUI or (AceGUI:GetWidgetVersion(Type) or 0) >= Version then return end

	-- Lua APIs
	local pairs, tsort = pairs, table.sort

	-- WoW APIs
	local _G = _G
	local PlaySound, CreateFrame, UIParent = PlaySound, CreateFrame, UIParent

	--[[-----------------------------------------------------------------------------
	Callbacks
	-------------------------------------------------------------------------------]]
	
	local function ItemSelected(self, selected)
		self.userdata.obj:SetValue(self.userdata.value)
	end

	--[[-----------------------------------------------------------------------------
	Scripts
	-------------------------------------------------------------------------------]]

	--[[-----------------------------------------------------------------------------
	Methods
	-------------------------------------------------------------------------------]]
	
	local function IterateMenues(self, value)
		local container = self.type == "FrameAnchorDropdown" and self.pullout or self.submenu
		for i, sub in container:IterateItems() do
			if sub.type == "Dropdown-Item-Toggle" then
				if sub.userdata.value ~= value then
					sub:SetValue(false)
				else
					sub:SetValue(true)
					sub.userdata.obj:SetText(table.concat({("\001"):split(sub.userdata.category)}, "->") .. ": " .. sub.userdata.name)
				end
			elseif sub.type == "Dropdown-Item-Menu" then
				IterateMenues(sub, value)
			end
		end
	end
	
	local function GetMenu(self, ...)
		local n = select("#", ...)
		local cat = ...
		local m
		
		local container = self.type == "FrameAnchorDropdown" and self.pullout or self.submenu		
		
		for i, menu in container:IterateItems() do
			if menu.userdata.value == cat then
				m = menu
				break
			end
		end
		
		if not m then
			m = AceGUI:Create("Dropdown-Item-Menu")
			m.userdata.value = cat
			m.SetValue = function() end
			m:SetText(cat)
			if m.submenu then
				m.submenu = nil
			end
			if not m.submenu then				
				local sub = AceGUI:Create("Dropdown-Pullout")
				m:SetMenu(sub)
				m.OnRelease = function(self)
					if self.submenu then
						self.submenu:Release()
					end
				end
			end
			container:AddItem(m)
		end
		
		if n > 1 then
			return GetMenu(m, select(2, ...))
		end
		
		return m
	end
	
	local sortlist = {}
	local function ListSort(a, b) 
		if a.category == b.category then
			return a.name < b.name
		else
			return a.category < b.category
		end
	end
	
	local methods = {
	
		["SetValue"] = function(self, value)
			if value then
				IterateMenues(self, value)
			end
			if self.open then	
				--self.pullout:Close()
			end
			self:Fire("OnValueChanged", value)
		end,		
		
		["SetList"] = function(self, list)			
			self.pullout:Clear()
			self.hasClose = nil
			
			wipe(sortlist)
			for i, d in pairs(list) do
				if type(d) == "table" then
					tinsert(sortlist, d)
				end
			end
			tsort(sortlist, ListSort)
			
			for i, data in ipairs(sortlist) do			
				self:AddItem(data)
			end
		end,		
		
		["AddItem"] = function(self, data)
			-- create submenu
			local m = GetMenu(self, ("\001"):split(data.category))			
			
			local subCheck = AceGUI:Create("Dropdown-Item-Toggle")
			subCheck.userdata.value = data.ident
			subCheck.userdata.category = data.category
			subCheck.userdata.name = data.name
			subCheck.userdata.obj = self
			subCheck:SetText(data.name)
			subCheck:SetDisabled(data.disabled)	
			subCheck:SetValue(false)
			subCheck:SetCallback("OnValueChanged", ItemSelected)
			m.submenu:AddItem(subCheck)
		end,
	}

	--[[-----------------------------------------------------------------------------
	Constructor
	-------------------------------------------------------------------------------]]
	local function Constructor()
		local count = AceGUI:GetNextWidgetNum(Type)
		local widget = AceGUI:Create("Dropdown")
		widget.type = Type
		
		for method, func in pairs(methods) do
			widget[method] = func
		end

		return AceGUI:RegisterAsWidget(widget)
	end

	AceGUI:RegisterWidgetType(Type, Constructor, Version)

end