local AceGUI = LibStub("AceGUI-3.0")

-- Lua APIs
local min, max, floor = math.min, math.max, math.floor
local tonumber = tonumber

-- WoW APIs
local CreateFrame, UIParent = CreateFrame, UIParent

-- Global vars/functions that we don't upvalue since they might get hooked, or upgraded
-- List them here for Mikk's FindGlobals script
-- GLOBALS: GameFontHighlightSmall

--------------------------
-- Slider  	            --
--------------------------
do
	local Type = "UIF-Slider"
	local Version = 4
	
	local function OnAcquire(self)
		self.OldOnAcquire(self)
		self.slider:EnableMouseWheel(true)
	end
	
	local function Slider_OnMouseWheel(frame, v)
		local self = frame.obj
		if not self.disabled and IsShiftKeyDown() then
			local value = self.value
			if v > 0 then
				value = min(value + (IsControlKeyDown() and 4 * self.step or self.step or 1), self.max)
			else
				value = max(value - (IsControlKeyDown() and 4 * self.step or self.step or 1), self.min)
			end
			self.slider:SetValue(value)
		end
	end
	
	local function Frame_OnEvent(self, ...)
		if IsShiftKeyDown() then
			self.obj.slider:SetScript("OnMouseWheel", Slider_OnMouseWheel)
		else
			self.obj.slider:SetScript("OnMouseWheel", nil)
		end
	end
	
	local function SetDisabled(self, disabled)
		self.oldSetDisable(self, disabled)
		if not disabled and self.newColor then
			self.label:SetTextColor(unpack(self.newColor))
		self.lowtext:SetTextColor(unpack(self.newColor))
		self.hightext:SetTextColor(unpack(self.newColor))
		end
	end
	
	local function SetFont(self, font, size, flags)
		local f, s, fl = GameFontNormal:GetFont()
		size, flags = size or s, flags or fl
		self.label:SetFont(font, size, flags)
		self.lowtext:SetFont(font, size, flags)
		self.hightext:SetFont(font, size, flags)
		self.editbox:SetFont(font, size, "none")
	end
	
	local function SetFontColor(self, r, g, b, a)
		if type(r) == "table" then
			r, g, b, a = unpack(r)
		end
		self.newColor = {r, g, b, a}
		self.label:SetTextColor(r, g, b, a)
		self.lowtext:SetTextColor(r, g, b, a)
		self.hightext:SetTextColor(r, g, b, a)
	end	
	
	local function Constructor()
		local self = AceGUI:Create("Slider")
		self.type = Type
		
		self.oldSetDisable = self.SetDisabled
		self.SetDisabled = SetDisabled

		self.SetFont = SetFont
		self.SetFontColor = SetFontColor
		
		self.OldOnAcquire = self.OnAcquire
		self.OnAcquire = OnAcquire
		
		self.frame:RegisterEvent("MODIFIER_STATE_CHANGED")
		self.frame:SetScript("OnEvent", Frame_OnEvent)
		self.slider:SetScript("OnMouseWheel", nil)
		--self.slider:SetScript("OnMouseWheel", Slider_OnMouseWheel)
		
		return self
	end
	
	AceGUI:RegisterWidgetType(Type,Constructor,Version)
end
