local mod = RW:Boss("Atramedes", "Blackwing Descent")
local L

mod.BossIcon = [[Interface\ICONS\achievement_dungeon_blackwingdescent_raid_atramedes]]

---------------------------
-- Variables

---------------------------
-- Init

function mod:OnRegister()
	L = mod:RegisterLocale(self:GetName(), "enUS", true)
	if L then
		L["Air Phase Emote"] = "Yes, run! With every step your heart quickens. The beating, loud and thunderous... Almost deafening. You cannot escape!"
		
		L["Air Phase"] = true
		L["Ground Phase"] = true
		L["Phase Changes"] = true
		
		L["Obnoxious Fiend up!"] = true
		L["Obnoxious Fiend"] = true
	end
	L = mod:GetLocale()
	self.L = L
	
	self:SetTrigger(41442)
	self:SetWin()
	
	-- Air Phase
	self:RegisterAlert("wAirPhase", self:CustomWarning(L["Air Phase"], L["Air Phase"], [[Interface\Vehicles\UI-Vehicles-Button-Pitch-Up]], L["Phase Changes"]))
	self:RegisterAlert("tAirPhase", self:CustomTimer(90, L["Air Phase"], L["Air Phase"], [[Interface\Vehicles\UI-Vehicles-Button-Pitch-Up]], L["Phase Changes"]))
	
	-- Ground Phase
	self:RegisterAlert("wGroundPhase", self:CustomWarning(L["Ground Phase"], L["Ground Phase"], [[Interface\Vehicles\UI-VEHICLES-BUTTON-PITCHDOWN-UP]], L["Phase Changes"]))
	self:RegisterAlert("tGroundPhase", self:CustomTimer(33, L["Ground Phase"], L["Ground Phase"], [[Interface\Vehicles\UI-VEHICLES-BUTTON-PITCHDOWN-UP]], L["Phase Changes"]))
	
	-- Sonic Breath
	self:RegisterAlert("wSonic", self:CastWarning(78075))
	self:RegisterAlert("tSonicCD", self:CDTimer(41, 78075))
	
	-- Searing Flame
	self:RegisterAlert("wFlames", self:CastWarning(77840))
	self:RegisterAlert("tFlamesCD", self:CDTimer(45, 77840))
	-- Shields
	
	-- Tracking
	self:RegisterAlert("iTracking", self:Icon("SKULL", 78092, false, 15))
	self:RegisterAlert("wTracking", self:RunawayWarning(78092))
	self:RegisterAlert("mTracking", self:TargetMessage(78092))
		
	self:RegisterAlert("wFiend", self:Warning(L["Obnoxious Fiend up!"], 92681, L["Obnoxious Fiend"]))
	self:RegisterAlert("iFiend", self:Icon("CROSS", L["Obnoxious Fiend"], false, 5))
	
end

---------------------------
-- Locals
local imps = {}

---------------------------
-- Start

function mod:OnEngage(trigger)
	self:Track("SonicBreath", 78075)
	self:Track("SearingFlames", 77840, 77982, 92421, 92422, 92423)
	self:Track("TrackTarget", 78092)
	self:Track("PhaseShift", 92681)
	
	self:YellTrack("AirPhase", L["Air Phase Emote"])
	
	wipe(imps)
	
	self:Call("tSonicCD", "Start", 25)
	self:Call("tFlamesCD", "Start")
	self:Call("tAirPhase", "Start")
end

function mod:SonicBreath(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		self:Call("wSonic", "Show")
		self:Call("tSonicCD", "Start")
	end
end

function mod:SearingFlames(event, args)
	if event == "SPELL_AURA_APPLIED" then
		self:Call("wFlames", "Show")
		self:Call("tFlamesCD", "Start")
	end
end

function mod:PhaseShift(event, args)
	if event == "SPELL_AURA_APPLIED" then
		if not imps[args.dGUID] then
			imps[args.dGUID] = true
			self:Call("wFiend", "Show")
		end
	end
end

function mod:SWING_DAMAGE(event, args)
	if self:CUIDFromGUID(args.sGUID) == 49740 then
		self:Call("iFiend", "Show", args.dName)
	end
end

local function StartGroundPhase()
	mod:Call("wGroundPhase", "Show")
	mod:Call("tAirPhase", "Start")
	mod:Call("tFlamesCD", "Start")
	mod:Call("tSonicCD", "Start", 25)
end

function mod:AirPhase(event, msg, name)
	self:Call("tSonicCD", "Stop")
	self:Call("tFlamesCD", "Stop")
	self:Call("tAirPhase", "Stop")
	
	self:Call("tGroundPhase", "Start")
	self:Call("wAirPhase", "Show")	
	
	self:Schedule("GroundPhase", 33, StartGroundPhase)
end

function mod:TrackTarget(event, args)
	if event == "SPELL_AURA_APPLIED" then
		self:Call("iTracking", "Show", args.dName)
		self:Call("mTracking", "Show", args.dName)
		
		if args:IsPlayer() then
			self:Call("wTracking", "Show")
		end
	end
end
