local mod = RW:Boss("Omnotron Defense System", "Blackwing Descent")
local L

mod.BossIcon = [[Interface\ICONS\achievement_dungeon_blackwingdescent_darkironcouncil]]
mod.SubCatIcon = [[Interface\ICONS\achievement_boss_nefarion]]

---------------------------
-- Variables

-- Electron	
	-- Conducor
	local iConductor, wConductor, tConductorCD
	-- Unstable Shield
	local tUnstableShieldDur, wUnstableShield
		
-- Arcanotron
	-- Power Conversion
	local tPowerConversionDur, wPowerConversion
	-- Power Generator
	local tPowerGeneratorCD, wPowerGenerator

-- Magmatron
	-- Acquire
	local iAquire, wAcquire, tAcquireCD
	-- Barrier
	local tBarrierDur, wBarrier

-- Toxitron
	-- Fixate
	local wFixate, iFixate, mFixate
	-- Poison Protocol
	local wProtocol, tProtocolCD
	-- Shell
	local wShell, tShellDur
	-- Bomb
	local wBomb, tBombCD
	
-- Active
local tActiveCD, wActiveChanged


-- Electron:	42179
-- Arcanotron:	42166
-- Magmatron:	42178
-- Toxitron:	42180

local Electron 		= 42179
local Arcanotron	= 42166
local Magmatron		= 42178
local Toxitron		= 42180
local GolemLookup = {}

---------------------------
-- Init

function mod:OnRegister()
	L = mod:RegisterLocale(self:GetName(), "enUS", true)
	if L then
		L["Electron"] = true
		L["Arcanotron"] = true
		L["Magmatron"] = true
		L["Toxitron"] = true
		L["Nefarian"] = true
	
		L["%s Activated!"] = true
		L["Warn when a golem Activates"] = true
		L["%s Activates"] = true
		L["Timer for when next specific golem Activates"] = true
		
		L["Stop hitting %s!"] = true
		L["Warn when to stop dps on target"] = true
		
		L["Next Nefairan ability"] = true
		L["Timer for next Nefarian ability"] = true
		L["Nefarian Abilities"] = true
	end
	L = mod:GetLocale()
	self.L = L
	
	self:SetTrigger(42178, 42180, 42179, 42166)
	self:SetWin()
	
	GolemLookup[Electron] = L["Electron"]
	GolemLookup[Arcanotron] = L["Arcanotron"]
	GolemLookup[Magmatron] = L["Magmatron"]
	GolemLookup[Toxitron] = L["Toxitron"]
	
	self:AddToCategory(L["Electron"], 79888, 79900)
	self:AddToCategory(L["Arcanotron"], 79729, 79624)
	self:AddToCategory(L["Magmatron"], 79501, 79582, 91521)
	self:AddToCategory(L["Toxitron"], 80094, 80053, 79835, 80157, 80161)
	self:AddToCategory(L["Nefarian"], 92048, 92053, 92023, 91849, 91857, 91879)
	
	-- Based on Video/Logs every 51 secs a golem goes from active -> shielded
	-- And appears that only 2 golems are up at any given time
	
	-- Electron	
		-- Conducor
		iConductor		= self:Icon("CROSS", 79888, false, 10)
		wConductor		= self:RunawayWarning(79888)
		tConductorCD	= self:CDTimer(27, 79888)
		self:RegisterAlert("mConductor", self:TargetMessage(79888))
		-- Unstable Shield
		tUnstableShieldDur	= self:DurTimer(10, 79900)
		wUnstableShield		= self:CustomWarning(L["Stop hitting %s!"], L["Warn when to stop dps on target"], 79900)
	
	-- Arcanotron
		-- Power Conversion
		wPowerConversion	= self:CustomWarning(L["Stop hitting %s!"], L["Warn when to stop dps on target"], 79729)
		tPowerConversionDur	= self:DurTimer(10, 79729)
		-- Power Generator
		wPowerGenerator		= self:CastWarning(79624)
		tPowerGeneratorCD	= self:CDTimer(30, 79624)
	
	-- Magmatron
		-- Acquire (Flame)
		iAquire			= self:Icon("SQUARE", 79501, false, 10)
		wAcquire		= self:RunawayWarning(79501)
		tAcquireCD		= self:CDTimer(40, 79501)		
		-- Barrier
		wBarrier		= self:CustomWarning(L["Stop hitting %s!"], L["Warn when to stop dps on target"], 79582)
		tBarrierDur		= self:DurTimer(10, 79582)
		--Incineration Security Measure
		self:RegisterAlert("tSecurityCD", self:CDTimer(27, 91521))
	
	-- Toxitron
		-- Fixate
		wFixate			= self:RunawayWarning(80094)
		iFixate			= self:Icon("STAR", 80094, false, 10, "CIRCLE", "SQUARE")
		mFixate			= self:TargetMessage(80094)		
		-- Poison Protocol
		wProtocol		= self:CastWarning(80053)
		tProtocolCD		= self:CDTimer(45, 80053)
		-- Poison Soaked Shell
		wShell			= self:CustomWarning(L["Stop hitting %s!"], L["Warn when to stop dps on target"], 79835)
		tShellDur		= self:DurTimer(10, 79835)
		-- Bomb
		wBomb			= self:CastWarning(80157)
		tBombCD			= self:CDTimer(30, 80157)
		self:RegisterAlert("wCloud", self:RunawayWarning(80161))
	
	-- Nef
		-- Shadow Infusion
		self:RegisterAlert("wInfusion", self:Warning(92048))
		self:RegisterAlert("wInfusionOnYou", self:RunawayWarning(92048))
		self:RegisterAlert("iInfusion", self:Icon("SKULL", 92048, false, 15))
		self:RegisterAlert("dInfusion", self:DirectionTo(92048))
		
		self:RegisterAlert("tShaodwConductorDur", self:DurTargetTimer(10, 92053))
		
		-- Encasing Shadows
		self:RegisterAlert("wEncasing", self:RunawayWarning(92023))
		
		-- Grip
		self:RegisterAlert("wGrip", self:CastWarning(91849))
		self:RegisterAlert("tGripCast", self:CastTimer(2, 91849))
		
		-- Overcharged Generator
		self:RegisterAlert("wOvercharged", self:CastWarning(91857))
		self:RegisterAlert("tBlowBack", self:CastTimer(8, 91879))
		
		self:RegisterAlert("tNextNef", self:CustomTimer(35, L["Next Nefairan ability"], L["Timer for next Nefarian ability"], 92023, L["Nefarian Abilities"]))
	
	-- Active
	tActiveCD		= self:CDTimer(45, 78740)
	self:RegisterAlert("tActiveSpecificCD", self:CustomTimer(45, L["%s Activates"], L["Timer for when next specific golem Activates"], 78740))
	wActiveChanged	= self:CustomWarning(L["%s Activated!"], L["Warn when a golem Activates"], 78740)
end

---------------------------
-- Locals
local fixateSpam = {}
local GolemOrder = {}
local FoundGolems = {}
local CurrentGolem
local CurrentGolemID
local BarrierLeft, BarrierTotal

local AcquireName = GetSpellInfo(79501)
local ConductorName = GetSpellInfo(79888)
local BombName = GetSpellInfo(80157)
local BarrierSpell = GetSpellInfo(91518)

---------------------------
-- Start

function mod:OnEngage(trigger)
	-- Electron
	self:Track("Conductor", 79888, 91433, 91432, 91431)
	self:Track("UnstableShield", 79900, 91448, 91449, 91447)
	
	-- Arcanotron
	self:Track("PowerConversion", 79729, 91545, 91544, 91543)
	self:Track("PowerGenerator", 79624)
	
	-- Magmatron
	self:TrackAuraApplied("Acquire", 79501, 92037, 92035, 92036)
	self:Track("Barrier", 79582, 91518, 91516, 91517)
	self:Track("Security", 79023, 91521, 91519, 91520)
	
	-- Toxitron	
	self:Track("Fixate", 80094)
	self:Track("Protocol", 80053, 91514, 91515, 91513, 91499)
	self:Track("Shell", 79835, 91502, 91503, 91501)
	self:Track("ChemicalBomb", 80157)
	self:Track("ChemicalBombAura", 80161, 91480, 91479, 91473, 91471)
	
	-- Nef
	self:Track("Infusion", 92048)
	self:Track("ShadowConductor", 92053)
	self:Track("Encasing", 92023)
	self:Track("Grip", 91849)
	self:Track("Overcharge", 91857)
	
	self:Track("GolemActivated", 78740, 95016, 95017, 95018)
	
	wipe(fixateSpam)
	wipe(GolemOrder)
	wipe(FoundGolems)
	CurrentGolem = 1
	CurrentGolemID = 0
	
	local cid = self:CUIDFromUID("boss1")
	if cid then
		tinsert(GolemOrder, cid)
	end
	
	if self:InstanceSize() == 25 then
		BarrierTotal = 450000
	else
		BarrierTotal = 150000
	end
	
	if self:IsHeroic() then
		tActiveCD:Start(30)
		self:Call("tNextNef", "Start", 20)
	else
		tActiveCD:Start()
	end
end


-- Electron
do
	function mod:Conductor(event, args) 
		if event == "SPELL_CAST_SUCCESS" then
			if args:IsPlayer() then
				wConductor:Show()
				SendChatMessage(ConductorName.. " on "..self.pName.."!", "SAY")
			end
			iConductor:Show(args.dName)
			self:Call("mConductor", "Show", args.dName)
			if self:IsHeroic() then
				tConductorCD:Start(20)
			else
				tConductorCD:Start()
			end
		end
	end
	
	function mod:UnstableShield(event, args)
		if event == "SPELL_AURA_APPLIED" then
			tUnstableShieldDur:Start()
		
		elseif event == "SPELL_CAST_START" then
			if UnitGUID("target") == args.sGUID and RW.Utils:GetRole() ~= "healer" then
				wUnstableShield:Show(args.sName)
			end
		end
	end
end

-- Arcanotron
do
	function mod:PowerConversion(event, args)
		if event == "SPELL_AURA_APPLIED" then
			tPowerConversionDur:Start()
		
		elseif event == "SPELL_CAST_START" then
			if UnitGUID("target") == args.sGUID and RW.Utils:GetRole() ~= "healer" then
				wPowerConversion:Show(args.sName)
			end
		end
	end
	
	function mod:PowerGenerator(event, args)
		if event == "SPELL_CAST_SUCCESS" then
			wPowerGenerator:Show()
			if self:IsHeroic() then
				tPowerGeneratorCD:Start(20)
			else
				tPowerGeneratorCD:Start()
			end
		end
	end
end

-- Magmatron
do
	function mod:Acquire(event, args)
		if args:IsPlayer() then
			wAcquire:Show()
			SendChatMessage(AcquireName.. " on "..self.pName.."!", "SAY")
		end
		iAquire:Show(args.dName)
		if self:IsHeroic() then
			tAcquireCD:Start(28)
		else
			tAcquireCD:Start()
		end
	end
	
	local function GetBarrierInfo()
		return BarrierLeft, BarrierTotal, BarrierSpell
	end
	
	function mod:Barrier(event, args)
		if event == "SPELL_AURA_APPLIED" then
			tBarrierDur:Start()
			
			BarrierLeft = BarrierTotal
			RW.Callbacks:Fire("MPBExternal", "BarrierHealth", GetBarrierInfo, nil, nil, select(3, GetSpellInfo(91517)))
		
		elseif event == "SPELL_CAST_START" then
			if UnitGUID("target") == args.sGUID and RW.Utils:GetRole() ~= "healer" then
				wBarrier:Show(args.sName)
			end
		
		elseif event == "SPELL_AURA_REMOVED" then
			RW.Callbacks:Fire("MPBExternalHide", "BarrierHealth")
		end
	end
	
	function mod:SPELL_MISSED(event, args)
		if (args.missType and args.missType == "ABSORB") and (args.amountMissed and args.amountMissed > 0) then
			if args:GetTargetCID() == Magmatron then
				BarrierLeft = BarrierLeft - args.amountMissed
			end
		end
	end
	
	function mod:SPELL_PERIODIC_MISSED(event, args)
		if (args.missType and args.missType == "ABSORB") and (args.amountMissed and args.amountMissed > 0) then
			if args:GetTargetCID() == Magmatron then
				BarrierLeft = BarrierLeft - args.amountMissed
			end
		end
	end
	
	function mod:SWING_MISSED(event, args)
		if (args.missType and args.missType == "ABSORB") and (args.amountMissed and args.amountMissed > 0) then
			if args:GetTargetCID() == Magmatron then
				BarrierLeft = BarrierLeft - args.amountMissed
			end
		end
	end
	
	function mod:RANGE_MISSED(event, args)
		if (args.missType and args.missType == "ABSORB") and (args.amountMissed and args.amountMissed > 0) then
			if args:GetTargetCID() == Magmatron then
				BarrierLeft = BarrierLeft - args.amountMissed
			end
		end
	end
	
	function mod:Security(event, args)
		if event == "SPELL_CAST_START" then
			self:Call("tSecurityCD", "Start")
		end
	end
end

-- Toxitron
do
	local function ClearFixate(name)
		fixateSpam[name] = nil
	end
	function mod:Fixate(event, args)
		if event == "SPELL_CAST_SUCCESS" then
			if not fixateSpam[args.dName] then
				fixateSpam[args.dName] = true
				self:Schedule("FixateSpam"..args.dName, 5, ClearFixate, args.dName)
				
				if args:IsPlayer() then
					wFixate:Show()
				end
				
				iFixate:Show(args.dName)
				mFixate:AddDelayTarget(args.dName)
				mFixate:DelayedShow(1)
			end
		end
	end
	
	function mod:Protocol(event, args)
		if event == "SPELL_CAST_SUCCESS" then
			if self:IsHeroic() then
				tProtocolCD:Start(25)
			else
				tProtocolCD:Start()
			end
			wProtocol:Show()
		end
	end
	
	function mod:Shell(event, args)	
		if event == "SPELL_AURA_APPLIED" then
			tShellDur:Start()
		
		elseif event == "SPELL_CAST_START" then
			if UnitGUID("target") == args.sGUID and RW.Utils:GetRole() ~= "healer" then
				wShell:Show(args.sName)
			end
		end
	end
	
	function mod:ChemicalBombTarget()
		local target = self:GetMobTarget(Toxitron)
		if target then
			if target == self.pName then
				SendChatMessage(BombName.. " on "..self.pName.."!", "SAY")
			end
		end
	end
	
	function mod:ChemicalBomb(event, args)
		if event == "SPELL_CAST_SUCCESS" then
			wBomb:Show()
			tBombCD:Start()
			
			self:Schedule("ChemicalBombTargetSchedule", 0.1, "ChemicalBombTarget")
		end
	end
	
	local lastClodWarn = 0
	function mod:ChemicalBombAura(event, args)
		if event == "SPELL_AURA_APPLIED" then
			if args:IsPlayer() then
				local t = GetTime()
				if t - lastClodWarn > 2 then
					self:Call("wCloud", "Show")
					lastClodWarn = t
				end
			end
		end
	end
end

-- Nef
do
	function mod:Infusion(event, args)
		if event == "SPELL_AURA_APPLIED" then
			if args:IsPlayer() then
				self:Call("wInfusionOnYou", "Show")
			else
				self:Call("wInfusion", "Show")
				self:Call("dInfusion", "Show", args.dName, 5)
			end
			
			self:Call("iInfusion", "Show", args.dName)
			self:Call("tNextNef", "Start")
		end
	end
	
	function mod:ShadowConductor(event, args)
		if event == "SPELL_AURA_APPLIED" then
			self:Call("tShaodwConductorDur", "Start", args.dName)
			
		elseif event == "SPELL_AURA_REMOVED" then
			self:Call("dInfusion", "Hide", args.dName)
		end
	end
	
	function mod:Encasing(event, args)
		if event == "SPELL_AURA_APPLIED" then
			if args:IsPlayer() then
			else
				self:Call("wEncasing", "Show")
			end
			
			self:Call("tNextNef", "Start")
		end
	end
	
	function mod:Grip(event, args)
		if event == "SPELL_CAST_START" then
			self:Call("wGrip", "Show")
			self:Call("tGripCast", "Start")
			self:Call("tNextNef", "Start")
		end
	end
	
	function mod:Overcharge(event, args)
		if event == "SPELL_CAST_SUCCESS" then
			self:Call("wOvercharged", "Show")
			self:Call("tBlowBack", "Start")
			self:Call("tNextNef", "Start")
		end
	end
end

-- Tox: 0xF150A4C4000001F6
-- Mag: 0xF150A4C2000001F4
-- Ele: 0xF150A4C3000001F3
-- Arc: 0xF150A4B6000001F5

function mod:GolemActivated(event, args)
	if event == "SPELL_AURA_APPLIED" then
		local cID = self:CUIDFromGUID(args.dGUID)
		
		if CurrentGolemID ~= cID then
			if CurrentGolem == 4 then
				CurrentGolem = 1
			else
				CurrentGolem = CurrentGolem + 1
			end
			CurrentGolemID = cID
			wActiveChanged:Show(args.dName)
		end		
		
		local nextGolem
		if CurrentGolem == 4 then
			nextGolem = GolemLookup[GolemOrder[1]]
		else
			nextGolem = GolemLookup[GolemOrder[CurrentGolem + 1]]
		end
		
		if #GolemOrder < 4 then
			if not FoundGolems[cID] then
				tinsert(GolemOrder, cID)
				FoundGolems[cID] = true
			end
			if #GolemOrder == 4 then
				if self:IsHeroic() then
					self:Call("tActiveSpecificCD", "Start", 30, nextGolem)
				else
					self:Call("tActiveSpecificCD", "Start", nextGolem)
				end
			else
				if self:IsHeroic() then
					tActiveCD:Start(30)
				else
					tActiveCD:Start()
				end
			end
		else
			if self:IsHeroic() then
				self:Call("tActiveSpecificCD", "Start", 30, nextGolem)
			else
				self:Call("tActiveSpecificCD", "Start", nextGolem)
			end
		end
		
		if cID == Electron then	
			tConductorCD:Start(15)
		
		elseif cID == Arcanotron then
			tPowerGeneratorCD:Start(16)
			
		elseif cID == Magmatron then
			tAcquireCD:Start(20)
			self:Call("tSecurityCD", "Start", 10)
			
		elseif cID == Toxitron then
			if self:IsHeroic() then
				tProtocolCD:Start(15)
				tBombCD:Start(25)
			else
				tProtocolCD:Start(20)
				tBombCD:Start(10)
			end
		end
	
	elseif event == "SPELL_AURA_REMOVED" then
		local cID = self:CUIDFromGUID(args.dGUID)
		
		if cID == Electron then	
			tConductorCD:Stop()
			
		elseif cID == Arcanotron then	
			tPowerGeneratorCD:Stop()
			
		elseif cID == Magmatron then	
			tAcquireCD:Stop()
			self:Call("tSecurityCD", "Stop")
			
		elseif cID == Toxitron then
			tProtocolCD:Stop()
			tBombCD:Stop()
		end
	end
end
