--[[
Plugin that handles flashing of the screen
]]

----------------------------------
-- Create plugin

local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("Player Alternate Power Bar", "AceHook-3.0", "AceTimer-3.0")
if not plugin then return end

----------------------------------
-- Locals 

local L = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Flash")
local LB = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Base")
local LSM = LibStub("LibSharedMedia-3.0")
local FA = LibStub("LibFrameAnchorRegistry-1.0")
local Utils = RW.Utils
local db
local flash = {}

local Anchor
local dragStart, dragStop

local RepeatSchedule, RepeatCheck

local defaults = {
	profile = {
		PluginEnabled = true,
		x = 0,
		y = 0,
		Point = "CENTER",
		RelPoint = "CENTER",
		AnchorFrame = nil,
		
		SpawnAnchor = "Default",
		
		HideDefault = false,
		ShowMulti = true,
	}
}

----------------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace("Player Alternate Power Bar", defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
	
	Anchor = Utils:CreateAnchor("AltPowerBarAnchor", UIParent, "Alternate Power Bar")
	Anchor:SetScript("OnDragStart", dragStart)
	Anchor:SetScript("OnDragStop", dragStop)	
end

function plugin:PluginEnable()
	if not db.PluginEnabled then
		self:Disable()
		return
	end
	
	self:RegisterCallback("ConfigMode")
	
	self:SecureHook(PlayerPowerBarAlt, "SetPoint", "PlayerPowerBarAltSetPoint")
	self:SecureHook(PlayerPowerBarAlt, "Show", "PlayerPowerBarAltShow")
	self:RegisterEvent("UNIT_POWER_BAR_SHOW")
	self:RegisterEvent("UNIT_POWER_BAR_HIDE")
	
	self:ScheduleTimer(function()
		if UnitAlternatePowerTextureInfo("player", 0) ~= nil then
			self:UNIT_POWER_BAR_SHOW("UNIT_POWER_BAR_SHOW", "player")
		end
	end, 0.5)
		
	RepeatSchedule = self:ScheduleRepeatingTimer(RepeatCheck, 2)
	
	self:Reset()
end

function plugin:PluginDisable()
	self:CancelTimer(RepeatSchedule, true)
	self:UnhookAll()
end

function plugin:Reset()
	db = self.db.profile
	
	local scale = Anchor:GetEffectiveScale() / UIParent:GetEffectiveScale()		
	local anchor = FA:GetAnchor(db.AnchorFrame or "BlizzUIParent")
	if not anchor then anchor = UIParent end	
	Anchor:ClearAllPoints()
	Anchor:SetPoint(db.Point, anchor, db.RelPoint, db.x / scale, db.y / scale)
		
	if UnitAlternatePowerTextureInfo("player", 0) ~= nil then
		plugin:UNIT_POWER_BAR_SHOW("UNIT_POWER_BAR_SHOW", "player")
		PlayerPowerBarAlt:Show()
	end
end

do
	local function getShieldHP()
		local barType, minPower, startInset, endInset, smooth, hideFromOthers, showOnRaid, opaqueSpark, opaqueFlash, powerName, powerTooltip = UnitAlternatePowerInfo("player")
		local max = UnitPowerMax("player", ALTERNATE_POWER_INDEX)
		local current = UnitPower("player", ALTERNATE_POWER_INDEX)
		return current or 0, max or 0, powerName or "Testing"
	end
	
	RepeatCheck = function()
		if UnitAlternatePowerTextureInfo("player", 0) ~= nil then
			plugin:UNIT_POWER_BAR_SHOW("UNIT_POWER_BAR_SHOW", "player")
		else
			RW.Callbacks:Fire("MPBExternalHide", "PlayerAlternatePowerBar")
		end
	end
	
	local SettingPosition
	
	function plugin:PlayerPowerBarAltSetPoint(frame, point, anchor, relPoint, x, y)
		if SettingPosition then return end
		SettingPosition = true
		frame:ClearAllPoints()
		frame:SetPoint("BOTTOM", Anchor, "TOP")
		SettingPosition = false
	end
	
	function plugin:PlayerPowerBarAltShow(frame)
		if UnitAlternatePowerTextureInfo("player", 0) then
			RW.Callbacks:Fire("MPBExternal", "PlayerAlternatePowerBar", getShieldHP, db.SpawnAnchor, nil, [[Interface\Icons\INV_Staff_53]])
		end
		if db.HideDefault then
			frame:Hide()
		end
	end


	function plugin:UNIT_POWER_BAR_SHOW(event, unit)
		if unit == "player" and db.ShowMulti then
			RW.Callbacks:Fire("MPBExternal", "PlayerAlternatePowerBar", getShieldHP, db.SpawnAnchor, nil, [[Interface\Icons\INV_Staff_53]])
		end
	end

end

function plugin:UNIT_POWER_BAR_HIDE(event, unit)	
	--RW.Callbacks:Fire("MPBExternalHide", "PlayerAlternatePowerBar")
end

-------------------------------------
-- Options

function plugin:ConfigMode(event, start)
	if start then
		Anchor:Show()
		if not PlayerPowerBarAlt:IsShown() then
			PlayerPowerBarAlt["frame"]:SetTexture("INTERFACE\\UNITPOWERBARALT\\MEAT_HORIZONTAL_FRAME.BLP")
			PlayerPowerBarAlt:SetSize(256, 64)
			PlayerPowerBarAlt:Show()
			PlayerPowerBarAlt.displayedValue = 0
			PlayerPowerBarAlt.value = 50
			PlayerPowerBarAlt.powerName = "Test"
			PlayerPowerBarAlt.smooth = true
			PlayerPowerBarAlt.range = 300
			PlayerPowerBarAlt.UpdateFill = UnitPowerBarAlt_Horizontal_UpdateFill
			PlayerPowerBarAlt.startInset = 0.13279999792576
			PlayerPowerBarAlt.endInset = 0.13670000433922
		end
		
	else
		Anchor:Hide()		
		if UnitAlternatePowerTextureInfo("player", 0) == nil then
			PlayerPowerBarAlt:Hide()
		end
	end
end

function plugin:GetOptions()	
	return  {{text = LB.PLUGINS, value = "Plugins"}, {text = "Alternate Power Bar", value = "PlayerAltPowerBar"}}, plugin.GetGUI, [[Interface\Icons\INV_Staff_53]]
end

local function Callback()
	plugin:Reset()
end

function plugin:GetGUI()
	local self = self or plugin
	local UIF = LibStub("LibGUIFactory-1.0"):GetFactory("RW")
	local AceGUI = LibStub("AceGUI-3.0")
	
	local group = AceGUI:Create("SimpleGroup")	
	group:SetFullWidth(true)
	group:SetLayout("Flow")
	
	group:AddChild(UIF:CheckBox("Hide Default", db, "HideDefault", Callback, "Hide the default blizzard alternate power bar", 0.5))
	group:AddChild(UIF:NewLine())
	group:AddChild(UIF:FrameAnchorDropdown("Anchor", db, "AnchorFrame", Anchor, Callback, 0.5))
	group:AddChild(UIF:NewLine())
	group:AddChild(UIF:GridSelect("Point", db, "Point", "Anchor", 3, Callback, 0.5, 75))
	group:AddChild(UIF:GridSelect("Relative Point", db, "RelPoint", "Anchor", 3, Callback, 0.5, 75))
	group:AddChild(UIF:PositionSlider("x", "X", db, "x", 1, Callback, 0.5))
	group:AddChild(UIF:PositionSlider("y", "Y", db, "y", 1, Callback, 0.5))
	
	local BarsPlugin = RW:GetPlugin("MultiPurposeBars")
	local Anchorlist
	if BarsPlugin then
		Anchorlist = BarsPlugin:GetAnchorList()
		
		if Anchorlist then
			group:AddChild(UIF:CheckBox("Show as bar via Multi-purose bar plugin", db, "ShowMulti", Callback, "Show the Alternate power bar as a normal bar on the multi-purpose bar anchor choosen.", 0.5))
			group:AddChild(UIF:Dropdown("Spaw at Multi-Purpose bar", db, "SpawnAnchor", Callback, Anchorlist, 0.5))
		end
	end
	
	return group
end

dragStart = function(self)
	self:StartMoving()
end
dragStop = function(self)
	self:StopMovingOrSizing()
	local scale = self:GetEffectiveScale() / UIParent:GetEffectiveScale()
	
	local x, y = self:GetCenter()
	x, y = x * scale, y * scale
	
	x = x - GetScreenWidth()/2
	y = y - GetScreenHeight()/2
	
	db.x = x
	db.y = y
	db.AnchorFrame = nil
	db.Point = "CENTER"
	db.RelPoint = "CENTER"
	
	RW.Callbacks:Fire("RefreshOptions")
end